// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package project

import (
	"context"
	"encoding/base64"
	"encoding/json"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"google.golang.org/grpc/status"
	"smart-enterprise-management/consts"
	"smart-enterprise-management/errors"
	"smart-enterprise-management/model"
	"smart-enterprise-management/pb/v1"
	"smart-enterprise-management/utils"
	"smart-enterprise-management/parser"
	"time"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"github.com/jaryhe/gopkgs/tasker/transtasker"
	"github.com/jaryhe/gopkgs/util"
)

func getB64Pass(passwd string) (string, error) {
	cryPasswd, err := util.AesEncrypt(passwd, CRYPTO_KEY)
	if err != nil {
		return "", errors.ParamsError
	}
	ret := base64.StdEncoding.EncodeToString(cryPasswd)
	return ret, nil
}

func checkProjectAddParam(req *v1.ProjectAddRequest) error {
	if req.CompanyId < 1{
		return errors.ParamsError
	}
	if req.Price <= 0 || req.Price > 9999999 {
		return status.Error(10003, "工程造价错误")
	}
	if req.Name == "" {
		return status.Error(10003, "项目名称不能为空")
	}
	if req.ProjectType == "" {
		return status.Error(10003, "项目类型不能为空")
	}
	if req.CoveredArea == 0 {
		return status.Error(10003, "建筑面积不能为空")
	}
	if req.StartDay == 0 {
		return status.Error(10003, "计划开工日期不能为空")
	}
	if req.EndDay == 0 {
		return status.Error(10003, "计划竣工日期不能为空")
	}
	if req.EndDay <= req.StartDay {
		return status.Error(10003, "计划竣工日期不能早于计划开工日期")
	}
	if req.ProjectFunction == "" {
		return status.Error(10003, "项目功能不能为空")
	}
	if req.TotalLevel == 0 {
		return status.Error(10003, "总层数不能为空")
	}
	if req.Phone == "" {
		return status.Error(10003, "电话不能为空")
	}
	if req.ProjectLeader == "" {
		return status.Error(10003, "项目负责人不能为空")
	}
	if req.SafetyRecordNo == "" {
		return status.Error(10003, "安监备案号不能为空")
	}
	if req.Construction == "" {
		return status.Error(10003, "施工单位不能为空")
	}
	if req.Development == "" {
		return status.Error(10003, "建筑单位不能为空")
	}
	if req.Oversee == "" {
		return status.Error(10003, "监理单位不能为空")
	}
	if req.Design == "" {
		return status.Error(10003, "设计单位不能为空")
	}
	if req.Exploration == "" {
		return status.Error(10003, "勘察单位不能为空")
	}
	if req.Lon == 0 {
		return status.Error(10003, "经度不能为空")
	}
	if req.Lat == 0 {
		return status.Error(10003, "纬度不能为空")
	}
	if req.Location == "" {
		return status.Error(10003, "项目所在地不能为空")
	}

	if !utils.VerifyMobileFormat(req.Phone) {
		return status.Error(10003, "业务负责人手机号格式错误")
	}
	return nil
}


func ProjectAdd(ctx context.Context, req *v1.ProjectAddRequest) (reply *v1.ProjectAddReply, err error) {
	if err := checkProjectAddParam(req); err != nil {
		return nil, err
	}
	now := time.Now()
	reply = &v1.ProjectAddReply{}
	addTask := func(db *gorm.DB) error {
		p := model.TProject{}
		where := map[string]interface{}{
			"safety_record_no":          req.SafetyRecordNo,
			"name":req.Name,
		}
		err = p.FindOr(db, nil, where)
		if err != nil && err != gorm.ErrRecordNotFound{
			logger.Error("ProjectAdd",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		if p.Id > 0 {
			return errors.ProjectExistError
		}

		p = model.TProject{}
		where = map[string]interface{}{
			"lon":  req.Lon,
			"lat":	req.Lat,
		}
		err = p.FindOr(db, where, nil)
		if err != nil && err != gorm.ErrRecordNotFound{
			logger.Error("ProjectAdd",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		if p.Id > 0 {
			return status.Error(20004, fmt.Sprintf("坐标%f,%f上已配置工地", req.Lon, req.Lat))
		}


		p.Cid = req.CompanyId
		//p.ProjectNo = req.ProjectNo
		p.Name = req.Name
		p.SafetyRecordNo = req.SafetyRecordNo
		p.ProjectLeader = req.ProjectLeader
		p.Phone = req.Phone
		p.Location = req.Location
		p.Name = req.Name
		p.TotalLevel = int64(req.TotalLevel)
		p.ProjectType = req.ProjectType
		if len(req.ProjectPlan) > 0 {
			bytes, _ := json.Marshal(req.ProjectPlan)
			p.ProjectPlan = string(bytes)
		}

		p.ProjectFunction = req.ProjectFunction
		p.Oversee = req.Oversee
		p.Exploration = req.Exploration
		p.Development = req.Development
		p.Design = req.Design
		p.Desc = req.Desc
		if len(req.ProjectEffectPic) > 0 {
			bytes, _ := json.Marshal(req.ProjectEffectPic)
			p.ProjectEffectPic = string(bytes)
		}
		p.CoveredArea = req.CoveredArea
		p.Construction = req.Construction
		p.BuildingStorey = req.BuildingStorey
		p.Price = req.Price
		p.Lat = req.Lat
		p.Lon = req.Lon
		p.UpdatedAt = now
		p.CreatedAt = now
		p.StartDay = time.Unix(req.StartDay, 0)
		p.EndDay = time.Unix(req.EndDay, 0)
		err = p.Insert(db)
		if err != nil {
			logger.Error("ProjectAdd",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		reply.Id = p.Id
		return nil
	}

	createProjectUserTask := func(db *gorm.DB) error {
		// 建设单位账号
		p := model.TSiteUser{
			Username:"jsdw"+fmt.Sprintf("%d", reply.Id),
			//Username:"c"+utils.GenerateRandomStr(3, "mix")+fmt.Sprintf("%d", reply.Id)+utils.GenerateRandomStr(3, "char"),
			UserType:consts.ProjectUserTypeConstruction,
			CreatedAt:now,
			UpdatedAt:now,
			Enable:true,
			ProjectId:reply.Id,
			SiteUser:0,
		}
		p.Passwd,_ = getB64Pass(utils.GenerateRandomStr(6, ""))
		err = p.Insert(db)
		if err != nil {
			logger.Error("ProjectAdd",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		// 项目部账号
		p.Id = 0
		p.Username = "xmb"+fmt.Sprintf("%d", reply.Id)
		p.Passwd,_ = getB64Pass(utils.GenerateRandomStr(6, ""))
		p.SiteUser = 1
		p.UserType = consts.ProjectUserTypeProject
		err = p.Insert(db)
		if err != nil {
			logger.Error("ProjectAdd",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		if parser.Conf.RunMode == "dev" {
			p.Id = 0
			p.Username = "admin"+fmt.Sprintf("%d", reply.Id)
			p.Passwd,_ = getB64Pass("123456")
			p.SiteUser = 1
			p.UserType = consts.ProjectUserTypeProject
			err = p.Insert(db)
			if err != nil {
				logger.Error("ProjectAdd",
					zap.String("err", err.Error()))
				return errors.DataBaseError
			}
		}

		// 监理部账号
		p.Id = 0
		p.Username = "jlb"+fmt.Sprintf("%d", reply.Id)
		p.Passwd,_ = getB64Pass(utils.GenerateRandomStr(6, ""))
		p.SiteUser = 0
		p.UserType = consts.ProjectUserTypeSupervision
		err = p.Insert(db)
		if err != nil {
			logger.Error("ProjectAdd",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		return nil
	}
	err = transtasker.Exec(database.DB(),
		transtasker.NewTask(addTask),
		transtasker.NewTask(createProjectUserTask))

	return reply, err


}
